;Moby version 20131201, A simple bootsector for the Master Boot Record, to load partition bootsectors.
;(C) Copyright 2013 Peter Wiehe (peter.wiehe@gmx.de), see mit-license.txt for copyright license.

		bits	16

		org	0

		;---Some BIOS' load to 0x7C0:0, others to 0:0x7C00. So unify it.
bbStart:	jmp	0x7C0:skip1
skip1:

		;---init stack
		mov	ax, 0x9000
		mov	ss, ax
		mov	sp, 0

		;--- copy this BB to 0x1000:0
copy:		push	word 0x7C0
		pop	ds
		push	word 0x1000
		pop	es
		mov	si, 0
		mov	di, 0
		mov	cx, 512
		rep movsb
		
		;--- jump to new location
evac:		jmp	0x1000:skip2
skip2:		push	word 0x1000
		pop	ds

		;---print message
		mov	si, bootMenuStr
		call	printChar

		;---read partition number
		call	readChar
		sub	al, '1'
		
		;---get first LBA from partition table
		mov	bx, 446			;start of partition table
		mov	ah, 0
		shl	ax, 4			;*16 = start of partition table entry
		add	bx, ax
		add	bx, 8			;1st LBA in entry
		mov	eax, [bx]
		mov	[diskAddrPackage.lba], eax
				
		;---load new BB
loadBB:		mov	si, diskAddrPackage
		mov	ah, 0x42
		mov	dl, 0x80
		int	0x13
		jc	loadBB
		
		;--- execute new BB
		push	word 0
		pop	ds
		mov	dl, 0x80
		jmp	0:0x7C00

;--------------- subroutines -----------------------------
readChar:	mov	ah, 0x00
		int	0x16
		call	printChar
		ret
		
printChar:	pusha
		;argument: al = character
		mov	bh, 0			;page #
		mov	bl, 7			;colour := grey
		mov	ah, 0x0E		;print char
		int	0x10
		popa
		ret
		
printString:	lodsb
		cmp	al, 0
		je	.end
		call	printChar
		jmp	short printString
.end:		ret

;--------------- data ------------------------------------
bootMenuStr:	db	"Moby 20131201", 13, 10, "Enter partition number (1-4)", 0

align 4
diskAddrPackage:
.site:		db	16
.zero:		db	0
.numOfSecs:	dw	1
.bufferOffset: 	dw	0
.bufferSegment:	dw	0x7C0
.lba:		dd	0

driveNumber:	db	0

;---------------------------------------------------------
times 218-($-$$) db 0
;several "extensions" to the MBR-partition-table

times 446-($-$$) db 0
partitionTable:	times 4*16 db 0

times 510-($-$$) db 0
magic:	db	0x55, 0xAA
end:
